from typing import ClassVar

from gisaf.models.geo_models_base import GeoPolygonModel
from gisaf.models.info_item import InfoItem

from ..metadata import tdc, avsm
# building_id_seq = db.Sequence('building_id_seq')


class Building(GeoPolygonModel, table=True):
    __tablename__ = 'building'
    __table_args__ = tdc.table_args

    class Admin:
        menu = 'Infrastructure'

    z_index: ClassVar[int] = 420
    icon: ClassVar[str] = 'other_tdc-building'
    symbol: ClassVar[str] = '\ue066'
    mapbox_type: ClassVar[str] = 'fill-extrusion'
    viewable_role: ClassVar[str] = 'tdc_source'

    # id = db.Column('id',
    #                db.Integer,
    #             #    building_id_seq,
    #                autoincrement=True,
    #             #    server_default=building_id_seq.next_value(),
    #                primary_key=True)
    build_type: str

    attribution: ClassVar[str] = 'ATDC'

    @property
    def caption(self):
        return '{self.build_type} (id {self.id})'.format(self=self)

    @classmethod
    async def get_popup(cls, df):
        return 'Building: ' + df['build_type'] + ' (id ' + df.index.astype(str) + ')'

    @classmethod
    async def get_properties(cls, df):
        return {
            'class': df['build_type'].str.replace(' ', '-')
        }

    async def get_feature_properties(self):
        ## DEPRECATED
        return {
            'class': self.build_type.replace(' ', '-')
        }

    async def get_info(self) -> list[InfoItem]:
        return [InfoItem(key='source', value="L'Avenir d'Auroville")]
