from typing import ClassVar
from sqlmodel import MetaData, Field

from gisaf.models.geo_models_base import GeoPolygonModel
from gisaf.models.info_item import InfoItem

from ...metadata import water_harvest, avsm

class CatchmentWatershed(GeoPolygonModel, table=True):
    __tablename__ = 'catchment_watershed'
    __table_args__ = water_harvest.table_args

    z_index: ClassVar[int] = 423

    id: int | None = Field(default=None, primary_key=True)

    skip_columns: ClassVar[set[str]] = set((
        'OBJECTID',
        'NAME',
        'CODE',
        'Shape_Leng',
        'Shape_Area',
        'Area_sq_km',
    ))
    error_on_import_bad_geometry_types: ClassVar[bool] = False

    async def get_info(self) -> list[InfoItem]:
        return [
            InfoItem(key='source', value='Water Harvest')
        ]

    attribution: ClassVar[str] = 'Water Harvest'