from typing import ClassVar
from sqlalchemy import String
from sqlmodel import Field

from gisaf.models.geo_models_base import GeoPolygonModel
from gisaf.models.info_item import InfoItem

from ..metadata import datameet

class Constituency(GeoPolygonModel, table=True):
    ## CREATE SCHEMA other_datameet;
    ## CREATE TABLE other_datameet.constituency (id serial not null PRIMARY KEY, name varchar(250), geom geometry(Polygon,4326));
    __tablename__ = 'constituency'
    __table_args__ = datameet.table_args

    class Admin:
        menu = 'Geo'
        flask_admin_model_view = 'MyModelViewWithPrimaryKey'

    z_index: ClassVar[int] = 407

    attribution: ClassVar[str] = 'Datameet'

    id: int | None = Field(default=None, primary_key=True)
    name: str = Field(sa_type=String(250)) # type: ignore

    def __str__(self):
        return self.name

    def __repr__(self):
        return f'<models.Constituency {self.name:s}>'

    synonyms: ClassVar[dict[str, str]] = {
        'PC_NAME': 'name',
    }

    @classmethod
    async def get_popup(cls, df):
        return 'Constituency: ' + df['name']

    @property
    def caption(self):
        return f'Constituency: {self.name}'

    @classmethod
    async def get_properties(cls, df):
        """
        Add a 'color' property based on the name
        """
        def get_color(name):
            hh = hash(name)
            return '#' + bytearray([(hh & 0xFF0000) >> 16, (hh & 0x00FF00) >> 8, hh & 0x0000FF]).hex()

        return {
            'color': df.name.apply(get_color)
        }

    async def get_info(self) -> list[InfoItem]:
        info = await super(Constituency, self).get_info()
        info.append(InfoItem(key='source', value='tameet (CC BY 2.5 IN)'))
        info.append(InfoItem(
            key='link',
            value='http://projects.datameet.org/maps/assembly-constituencies/'
        ))
        return info