from typing import ClassVar

from sqlmodel import Field

from gisaf.models.geo_models_base import GeoLineModelZ
from gisaf.models.info_item import InfoItem

from ..metadata import tdc

# contour_id_seq = db.Sequence('contour_id_seq')


class Contour(GeoLineModelZ, table=True):
    __tablename__ = 'contour'
    __table_args__ = tdc.table_args

    class Admin:
        menu = 'Geo'
        flask_admin_model_view = 'MyModelViewWithPrimaryKey'

    z_index: ClassVar[int] = 408
    id: int | None = Field(default=None, primary_key=True)
    elevation: float
    simplify: ClassVar[float] = 60

    icon: ClassVar[str] = 'other_tdc-contours'

    attribution: ClassVar[str] = 'ATDC'

    @classmethod
    async def get_popup(cls, df):
        return 'Elevation: ' + df['elevation'].astype('U') + ' m'

    @classmethod
    async def get_properties(cls, df):
        df['thick'] = (df['elevation'].mod(10) == 0).map({True: 1, False: 0})
        df['medium'] = (df['elevation'].mod(5) == 0).map({True: 1, False: 0})
        return {
            'width': df['thick'] + df['medium'] + 1,
            'label': df['elevation'].astype('U') + ' m',
        }

    @property
    def caption(self):
        return 'elevation: {self.elevation:.1f} m'.format(self=self)

    async def get_feature_properties(self):
        ## DEPRECATED
        if not self.elevation % 10:
            width = 3
        elif not self.elevation % 5:
            width = 2
        else:
            width = 1
        return {'width': width}

    async def get_info(self) -> list[InfoItem]:
        return [InfoItem(key='source', value="L'Avenir d'Auroville")]