from typing import ClassVar, Any
from sqlmodel import Field, Relationship

from gisaf.models.models_base import Model
from gisaf.models.geo_models_base import GeoPolygonModel
from gisaf.models.info_item import InfoItem

from ..metadata import misc


# geological_formation_type_id_seq = db.Sequence('geological_formation_type_id_seq')
# geological_formation_id_seq = db.Sequence('geological_formation_id_seq')


class GeologicalFormationType(Model, table=True):
    __tablename__ = 'geological_formation_type'
    __table_args__ = misc.table_args

    class Admin:
        menu = 'Geo'
        flask_admin_model_view = 'MyModelViewWithPrimaryKey'

    id: int | None = Field(default=None, primary_key=True)
    formation: str
    era: str
    period: str
    lithology: str
    minimum: str
    thickness: str
    maximum: str
    remarks: str
    components: str
    extra: str
    info: str
    reference: str
    aquifer: str

    def __str__(self):
        return self.formation

    def __repr__(self):
        return '<models.GeologicalFormationType {self.formation:s}>'.format(self=self)


class GeologicalFormation(GeoPolygonModel, table=True):
    __tablename__ = 'geological_formation'
    __table_args__ = misc.table_args

    class Admin:
        menu = 'Geo'

    z_index: ClassVar[int] = 405
    id: int | None = Field(default=None, primary_key=True)
    form_id: int = Field(foreign_key='avsm_misc.geological_formation_type.id')
    formation: GeologicalFormationType = Relationship()

    icon: ClassVar[str] = 'avsm_misc-geological_formation'

    @classmethod
    def selectinload(cls):
        return [cls.formation]

    # join_with: ClassVar[dict[str, Any]] = {
    #     'formation': GeologicalFormationType,
    # }

    @classmethod
    async def get_popup(cls, df):
        return 'Geological formation: ' + df['avsm_misc_geological_formation_type_formation']

    @classmethod
    async def get_properties(cls, df):
        return {
            'class': df['form_id'].to_numpy('U')
        }

    @property
    def caption(self):
        return self.formation.formation

    async def get_feature_properties(self):
        ## DEPRECATED
        return {
            'class': str(self.form_id)
        }

    async def get_info(self) -> list[InfoItem]:
        return [
            InfoItem(key='era', value=f'{self.formation.era}, {self.formation.period}'),
            InfoItem(key='lithology', value=self.formation.lithology),
            InfoItem(key='area', value=f'{self.area / 1000000:.1f} sq. km'),
            InfoItem(key='source', value="Auroville CSR Geomatics"),
        ]