from datetime import datetime
from typing import ClassVar
from sqlmodel import Field, SQLModel, String, MetaData
from gisaf.models.geo_models_base import GeoPointModel, GeoPointZModel, GeoLineModel
from gisaf.models.info_item import InfoItem

from ..metadata import aves

## Old layer - Johan's data
## TODO: delete
# class ElectricalLine(GeoLineModel):
#     __tablename__ = 'electrical_line'
#     metadata = metadata

#     icon = 'avsm_misc-electrical_line'

#     class Admin:
#         menu = 'Infrastructure'

#     id: int | None = Field(default=None, primary_key=True)
#     starts_at: str = Field(sa_type=String(25))
#     ends_at: str = Field(sa_type=String(25))
#     start_code: str = Field(sa_type=String(15))
#     end_code: str = Field(sa_type=String(15))
#     voltage: int
#     cores: int
#     core_size: str = Field(sa_type=String(10))

#     skip_columns: ClassVar[set[str]] = set((
#         'ID #',
#     ))

#     synonyms: ClassVar[dict[str, str]] = {
#         'ID #': 'id',
#         'Starts at': 'starts_at',
#         'Ends at': 'ends_at',
#         'Start Code': 'start_code',
#         'End Code': 'end_code',
#         'Voltage': 'voltage',
#         '# of cores': 'cores',
#         'Core size': 'core_size',
#     }

#     def __str__(self):
#         return f'{self.voltage} V, {self.starts_at} to {self.ends_at}'

#     def __repr__(self):
#         return f'<models.ElectricalLine {self.name:s}>'

#     @classmethod
#     async def get_popup(cls, df):
#         return 'Electrical line: ' + df['voltage'].astype('int64').astype('U') + 'V'

#     @property
#     def caption(self):
#         return f'Electrical line {self.voltage}V'

#     async def get_info(self):
#         info = await super(ElectricalLine, self).get_info()
#         info['start'] = self.starts_at
#         info['end'] = self.ends_at
#         info['start code'] = self.start_code
#         info['end code'] = self.end_code
#         info['voltage'] = self.voltage
#         info['cores'] = self.cores
#         info['core size'] = self.core_size
#         info['source'] = "Johan/Manikandan"
#         return info


class AVESLine(GeoLineModel):
    class Admin:
        menu = 'Infrastructure'

    icon: ClassVar[str] = 'avsm_misc-electrical_line'
    synonyms: ClassVar[dict[str, str]] = {
        'from': 'from_loc',
        'to': 'to_loc',
        'Surveyer': 'surveyor',
    }

    id: int | None = Field(default=None, primary_key=True)
    code: str = Field(sa_type=String(20)) # type: ignore
    type1: str
    type2: str
    #location1 = db.Column(db.String)
    #location2 = db.Column(db.String)
    un: str
    from_loc: str
    to_loc: str
    nbrofcores: float
    section: float
    #lenght = db.Column(db.Float)
    surveyedby: str
    #surveyor = db.Column(db.String)
    depth: str
    #offcet = db.Column(db.String)
    #commisdate = db.Column(db.String)
    #remarks = db.Column(db.String)

    def __str__(self):
        return f'{self.code}'

    def __repr__(self):
        return f'<models.AVESLine {self.code:s}>'

    attribution = 'AVES'

    @classmethod
    async def get_popup(cls, df):
        return 'AVES line: ' + df['code'].fillna('(no code)')

    @property
    def caption(self):
        return f'AVES line {self.code}'

    async def get_info(self) -> list[InfoItem]:
        info = await super(AVESLine, self).get_info()
        info.append(InfoItem(key='code', value=self.code))
        info.append(InfoItem(key='type1', value=self.type1))
        info.append(InfoItem(key='type2', value=self.type2))
        #info.append(InfoItem(key='location1', value=self.location1))
        #info.append(InfoItem(key='location2', value=self.location2))
        info.append(InfoItem(key='un', value=self.un))
        info.append(InfoItem(key='from_loc', value=self.from_loc))
        info.append(InfoItem(key='to_loc', value=self.to_loc))
        info.append(InfoItem(key='nbrofcores', value=self.nbrofcores))
        info.append(InfoItem(key='section', value=self.section))
        #info.append(InfoItem(key='lenght', value=self.lenght))
        info.append(InfoItem(key='surveyedby', value=self.surveyedby))
        #info.append(InfoItem(key='surveyor', value=self.surveyor))
        info.append(InfoItem(key='depth', value=self.depth))
        #info.append(InfoItem(key='offcet', value=self.offcet))
        #info.append(InfoItem(key='commisdate', value=self.commisdate))
        #info.append(InfoItem(key='remarks', value=self.remarks))
        info.append(InfoItem(key='source', value="Arnout@AVES"))
        return info


class AVESEquipment(SQLModel):
    class Admin:
        menu = 'Infrastructure'

    symbol: ClassVar[str] = '\ue082'
    attribution: ClassVar[str] = 'AVES'
    synonyms: ClassVar[dict[str, str]] = {
        'from': 'from_loc',
        'to': 'to_loc',
        'Surveyer': 'surveyor',
    }

    id: int | None = Field(default=None, primary_key=True)
    code: str = Field(sa_type=String(20)) # type: ignore
    type1: str
    type2: str
    location1: str
    #location2 = db.Column(db.String)
    p: int
    un: str
    manufdate: datetime
    surveyedby: str
    #surveyor = db.Column(db.String)
    #surveydate = db.Column(db.Date)
    maxload: float
    brand: str
    modelnr: str
    serialnbmr: str
    tdcbldgdid: str
    #commisdate = db.Column(db.String)
    #remarks = db.Column(db.String)

    def __str__(self):
        return f'{self.code}'

    def __repr__(self):
        return f'<models.AVESEquipment {self.code:s}>'

    @classmethod
    async def get_popup(cls, df):
        return 'AVES equipment: ' + df['code'].fillna('(no code)')

    @property
    def caption(self):
        return f'AVES equipment {self.code}'

    async def get_info(self):
        info = await super(AVESEquipment, self).get_info()
        info['code'] = self.code
        info['type1'] = self.type1
        info['type2'] = self.type2
        info['location1'] = self.location1
        #info['location2'] = self.location2
        #info['manufdate'] = self.manufdate
        info['surveyedby'] = self.surveyedby
        #info['surveyor'] = self.surveyor
        #info['surveydate'] = self.surveydate
        #info['maxload'] = self.maxload
        #info['brand'] = self.brand
        #info['modelnr'] = self.modelnr
        #info['serialnbmr'] = self.serialnbmr
        #info['tdcbldgdid'] = self.tdcbldgdid
        #info['commisdate'] = self.commisdate
        #info['remarks'] = self.remarks
        info['source'] = "Arnout@AVES"
        return info


class AVESLTLine(AVESLine, table=True):
    __tablename__ = 'lt_line'
    __table_args__ = aves.table_args

    @classmethod
    async def get_popup(cls, df):
        return 'AVES LT line: ' + df['code'].fillna('(no code)')


class AVESHTLine(AVESLine, table=True):
    __tablename__ = 'ht_line'
    __table_args__ = aves.table_args

    @classmethod
    async def get_popup(cls, df):
        return 'AVES HT line: ' + df['code'].fillna('(no code)')


class AVESLTEquipment(AVESEquipment, GeoPointZModel, table=True):
    __tablename__ = 'lt_equipment'
    __table_args__ = aves.table_args

    @classmethod
    async def get_popup(cls, df):
        return 'AVES LT equipment: ' + df['code'].fillna('(no code)')


class AVESHTEquipment(AVESEquipment, GeoPointModel, table=True):
    __tablename__ = 'ht_equipment'
    __table_args__ = aves.table_args

    @classmethod
    async def get_popup(cls, df):
        return 'AVES HT equipment: ' + df['code'].fillna('(no code)')