from typing import ClassVar

from sqlmodel import Field

from gisaf.models.geo_models_base import GeoLineModel
from gisaf.models.info_item import InfoItem

from ..metadata import tdc

# road_id_seq = db.Sequence('road_id_seq')


class Road(GeoLineModel, table=True):
    __tablename__ = 'road'
    __table_args__ = tdc.table_args

    class Admin:
        menu = 'Infrastructure'

    viewable_role: ClassVar[str] = 'tdc_source'
    icon: ClassVar[str] = 'other_tdc-road'
    attribution: ClassVar[str] = 'ATDC'
    z_index: ClassVar[int] = 420

    id: int | None = Field(default=None, primary_key=True)
    road_type: str

    async def get_feature_properties(self) -> dict[str, str]:
        ## DEPRECATED
        return {
            'class': self.road_type.replace(' ', '-')
        }

    @classmethod
    async def get_popup(cls, df) -> str:
        return 'Road: ' + df['road_type']

    @classmethod
    async def get_properties(cls, df):
        return {
            'class': df['road_type'].str.replace(' ', '-')
        }

    @property
    def caption(self) -> str:
        return f'Road {self.road_type}'

    async def get_info(self) -> list[InfoItem]:
        return [InfoItem(key='source', value="L'Avenir d'Auroville")]