from typing import ClassVar

from sqlmodel import Field

from gisaf.models.geo_models_base import GeoPolygonModel
from gisaf.models.info_item import InfoItem

from ..metadata import tdc, avsm

# zone_id_seq = db.Sequence('zone_id_seq')
# zone_of_force_id_seq = db.Sequence('zone_of_force_id_seq')


class Zone(GeoPolygonModel, table=True):
    __tablename__ = 'zone'
    __table_args__ = tdc.table_args

    class Admin:
        menu = 'Planning'

    z_index: ClassVar[int] = 410
    viewable_role: ClassVar[str] = 'tdc_source'
    icon: ClassVar[str] = 'other_tdc-zone'
    attribution: ClassVar[str] = 'ATDC'

    id: int | None = Field(default=None, primary_key=True)
    zone: str

    @property
    def caption(self):
        return self.zone

    @classmethod
    async def get_popup(cls, df):
        return 'Zone: ' + df['zone']

    @classmethod
    async def get_properties(cls, df):
        return {
            'class': df['zone'].str.replace(' ', '-')
        }

    @classmethod
    def get_options(cls):
        return {
            'stroke': False,
            'fillOpacity': 0.2,
        }

    async def get_info(self) -> list[InfoItem]:
        return [InfoItem(key='source', value="L'Avenir d'Auroville")]


class LineOfForce(GeoPolygonModel, table=True):
    __tablename__ = 'line_of_force'
    __table_args__ = tdc.table_args

    class Admin:
        menu = 'Planning'

    z_index: ClassVar[int] = 412
    viewable_role: ClassVar[str] = 'tdc_source'

    id: int | None = Field(default=None, primary_key=True)

    def __str__(self):
        return self.zone

    icon: ClassVar[str] = 'other_tdc-line_of_force'
    attribution: ClassVar[str] = 'ATDC'

    async def get_info(self) -> list[InfoItem]:
        return [InfoItem(key='source', value="L'Avenir d'Auroville")]