from sqlalchemy import and_

from gisaf.models.tags import Tags as TagsModel
from gisaf.utils import upsert_df
from gisaf.registry import registry
from gisaf.plugins import TagPlugin


integrity = TagPlugin(
    domain='integrity',
    key='surveyor',
    stores_by_re=[''],  ## Match everything
    action='Fix integrity',
    roles=['Fix integrity']
)

async def execute(request, store, id, value):
    tags = await TagsModel.get_df(
        where=and_(TagsModel.store==store, TagsModel.ref_id==id),
        geom_as_ewkt=True
    )
    tags['integrity:surveyor'] = tags.tags.apply(lambda row: row.get('integrity:surveyor'))

    ## Fix features

    model = registry.stores.loc[store, 'model']
    df = await model.get_df(where=model.id==id, geom_as_ewkt=True)
    df['srvyr_id'] = int(value)
    await upsert_df(df, model)

    ## Remove tag
    for row in tags.itertuples():
        if 'integrity:surveyor' in row.tags:
            del row.tags['integrity:surveyor']
    await upsert_df(tags, TagsModel)

    return 'Fixed'


#class FixIntegrity(ActionPlugin):
#    async def execute(self, request, features, params):
#        ## XXX: Ideally, a request on HSTORE should be used. Doesn't work out of the box, though.
#        filter = [and_(TagsModel.store==store, TagsModel.ref_id.in_(ids))
#                  for store, ids in features.items()]
#        tags = await TagsModel.get_df(where=or_(*filter), geom_as_ewkt=True)
#        tags['integrity:surveyor'] = tags.tags.apply(lambda row: row.get('integrity:surveyor'))
#        stores = features.keys()
#        dfs = {}
#        #surveyor_df = await Surveyor.get_df(where=Surveyor.id==conf.survey['default']['surveyor_id'])
#
#        total_fixed = 0
#        for store in features.keys():
#            model = registry.stores.loc[store, 'model']
#            ## Merge (inner join) to get only features with the integrity:surveyor => missing tag
#            dfs[store] = (await model.get_df(where=model.id.in_(features[store]), geom_as_ewkt=True))\
#                .merge(tags.loc[~tags['integrity:surveyor'].isna()], left_on='id', right_on='ref_id')
#            dfs[store]['srvyr_id'] = conf.survey['default']['surveyor_id']
#
#            await upsert_df(dfs[store], model)
#            total_fixed += len(dfs[store])
#            ## Remove tag
#            for row in tags.itertuples():
#                if 'integrity:surveyor' in row.tags:
#                    del row.tags['integrity:surveyor']
#            await upsert_df(tags, TagsModel)
#
#        results = ActionResults(
#            message=f'{total_fixed} features fixed',
#            actionResults=[],
#        )
#        return results
#
#
#fix = FixIntegrity(
#    name='Auto fix integrity issues',
#    stores_by_re=['avsm_survey.*'],
#    roles=['manager'],
#    params=[
#        ActionParam(
#            name='surveyor',
#            type='string',
#            dflt='avsm'
#        )
#    ]
#)